Utf8Error

Struct Utf8Error 

pub struct Utf8Error { /* private fields */ }
Errors which can occur when attempting to interpret a sequence of u8 as a string.

This structure exactly duplicates std::str::Utf8Error to work around the fact that its constructors are private.

Implementations

Source

impl Utf8Error

pub const fn valid_up_to(&self) -> usize

Returns the index in the given string up to which valid UTF-8 was verified.

pub const fn error_len(&self) -> Option<usize>

Provides more information about the failure:

  • None: the end of the input was reached unexpectedly. self.valid_up_to() is 1 to 3 bytes from the end of the input. If a byte stream (such as a file or a network socket) is being decoded incrementally, this could be a valid char whose UTF-8 byte sequence is spanning multiple chunks.

  • Some(len): an unexpected byte was encountered. The length provided is that of the invalid byte sequence that starts at the index given by valid_up_to().