pub struct VectorGuard<'a>(/* private fields */);Expand description
Guards against panics during unsafe rope mutation.
VectorGuard is returned from Rope::as_mut_vector. It implements
DerefMut<Target=Vector<u8>>. If a VectorGuard is dropped by panicking,
it will clear the vector to prevent observation of a rope containing invalid
UTF-8 after catching the panic.
Methods from Deref<Target = Vector<u8>>
Sourcepub fn is_empty(&self) -> bool
pub fn is_empty(&self) -> bool
Test whether a vector is empty.
Time: O(1)
Examples
let vec = vector!["Joe", "Mike", "Robert"];
assert_eq!(false, vec.is_empty());
assert_eq!(true, Vector::<i32>::new().is_empty());Sourcepub fn is_inline(&self) -> bool
pub fn is_inline(&self) -> bool
Test whether a vector is currently inlined.
Vectors small enough that their contents could be stored entirely inside
the space of std::mem::size_of::<Vector<A>>() bytes are stored inline on
the stack instead of allocating any chunks. This method returns true if
this vector is currently inlined, or false if it currently has chunks allocated
on the heap.
This may be useful in conjunction with ptr_eq(), which checks if
two vectors’ heap allocations are the same, and thus will never return true
for inlined vectors.
Time: O(1)
Sourcepub fn ptr_eq(&self, other: &Vector<A>) -> bool
pub fn ptr_eq(&self, other: &Vector<A>) -> bool
Test whether two vectors refer to the same content in memory.
This uses the following rules to determine equality:
- If the two sides are references to the same vector, return true.
- If the two sides are single chunk vectors pointing to the same chunk, return true.
- If the two sides are full trees pointing to the same chunks, return true.
This would return true if you’re comparing a vector to itself, or
if you’re comparing a vector to a fresh clone of itself. The exception to this is
if you’ve cloned an inline array (ie. an array with so few elements they can fit
inside the space a Vector allocates for its pointers, so there are no heap allocations
to compare).
Time: O(1)
Sourcepub fn leaves(&self) -> Chunks<'_, A>
pub fn leaves(&self) -> Chunks<'_, A>
Get an iterator over the leaf nodes of a vector.
This returns an iterator over the Chunks at the leaves of the
RRB tree. These are useful for efficient parallelisation of work on
the vector, but should not be used for basic iteration.
Time: O(1)
Sourcepub fn leaves_mut(&mut self) -> ChunksMut<'_, A>
pub fn leaves_mut(&mut self) -> ChunksMut<'_, A>
Get a mutable iterator over the leaf nodes of a vector.
This returns an iterator over the Chunks at the leaves of the
RRB tree. These are useful for efficient parallelisation of work on
the vector, but should not be used for basic iteration.
Time: O(1)
Sourcepub fn get(&self, index: usize) -> Option<&A>
pub fn get(&self, index: usize) -> Option<&A>
Get a reference to the value at index index in a vector.
Returns None if the index is out of bounds.
Time: O(log n)
Examples
let vec = vector!["Joe", "Mike", "Robert"];
assert_eq!(Some(&"Robert"), vec.get(2));
assert_eq!(None, vec.get(5));Sourcepub fn get_mut(&mut self, index: usize) -> Option<&mut A>
pub fn get_mut(&mut self, index: usize) -> Option<&mut A>
Get a mutable reference to the value at index index in a
vector.
Returns None if the index is out of bounds.
Time: O(log n)
Examples
let mut vec = vector!["Joe", "Mike", "Robert"];
{
let robert = vec.get_mut(2).unwrap();
assert_eq!(&mut "Robert", robert);
*robert = "Bjarne";
}
assert_eq!(vector!["Joe", "Mike", "Bjarne"], vec);Sourcepub fn front(&self) -> Option<&A>
pub fn front(&self) -> Option<&A>
Get the first element of a vector.
If the vector is empty, None is returned.
Time: O(log n)
Sourcepub fn front_mut(&mut self) -> Option<&mut A>
pub fn front_mut(&mut self) -> Option<&mut A>
Get a mutable reference to the first element of a vector.
If the vector is empty, None is returned.
Time: O(log n)
Sourcepub fn head(&self) -> Option<&A>
pub fn head(&self) -> Option<&A>
Get the first element of a vector.
If the vector is empty, None is returned.
This is an alias for the front method.
Time: O(log n)
Sourcepub fn back(&self) -> Option<&A>
pub fn back(&self) -> Option<&A>
Get the last element of a vector.
If the vector is empty, None is returned.
Time: O(log n)
Sourcepub fn back_mut(&mut self) -> Option<&mut A>
pub fn back_mut(&mut self) -> Option<&mut A>
Get a mutable reference to the last element of a vector.
If the vector is empty, None is returned.
Time: O(log n)
Sourcepub fn last(&self) -> Option<&A>
pub fn last(&self) -> Option<&A>
Get the last element of a vector.
If the vector is empty, None is returned.
This is an alias for the back method.
Time: O(log n)
Sourcepub fn index_of(&self, value: &A) -> Option<usize>where
A: PartialEq,
pub fn index_of(&self, value: &A) -> Option<usize>where
A: PartialEq,
Get the index of a given element in the vector.
Searches the vector for the first occurrence of a given value,
and returns the index of the value if it’s there. Otherwise,
it returns None.
Time: O(n)
Examples
let mut vec = vector![1, 2, 3, 4, 5];
assert_eq!(Some(2), vec.index_of(&3));
assert_eq!(None, vec.index_of(&31337));Sourcepub fn contains(&self, value: &A) -> boolwhere
A: PartialEq,
pub fn contains(&self, value: &A) -> boolwhere
A: PartialEq,
Test if a given element is in the vector.
Searches the vector for the first occurrence of a given value,
and returns true if it’s there. If it’s nowhere to be found
in the vector, it returns false.
Time: O(n)
Examples
let mut vec = vector![1, 2, 3, 4, 5];
assert_eq!(true, vec.contains(&3));
assert_eq!(false, vec.contains(&31337));Sourcepub fn clear(&mut self)
pub fn clear(&mut self)
Discard all elements from the vector.
This leaves you with an empty vector, and all elements that were previously inside it are dropped.
Time: O(n)
Sourcepub fn binary_search_by<F>(&self, f: F) -> Result<usize, usize>
pub fn binary_search_by<F>(&self, f: F) -> Result<usize, usize>
Binary search a sorted vector for a given element using a comparator function.
Assumes the vector has already been sorted using the same comparator
function, eg. by using sort_by.
If the value is found, it returns Ok(index) where index is the index
of the element. If the value isn’t found, it returns Err(index) where
index is the index at which the element would need to be inserted to
maintain sorted order.
Time: O(log n)
Sourcepub fn binary_search(&self, value: &A) -> Result<usize, usize>where
A: Ord,
pub fn binary_search(&self, value: &A) -> Result<usize, usize>where
A: Ord,
Binary search a sorted vector for a given element.
If the value is found, it returns Ok(index) where index is the index
of the element. If the value isn’t found, it returns Err(index) where
index is the index at which the element would need to be inserted to
maintain sorted order.
Time: O(log n)
Sourcepub fn binary_search_by_key<B, F>(&self, b: &B, f: F) -> Result<usize, usize>
pub fn binary_search_by_key<B, F>(&self, b: &B, f: F) -> Result<usize, usize>
Binary search a sorted vector for a given element with a key extract function.
Assumes the vector has already been sorted using the same key extract
function, eg. by using sort_by_key.
If the value is found, it returns Ok(index) where index is the index
of the element. If the value isn’t found, it returns Err(index) where
index is the index at which the element would need to be inserted to
maintain sorted order.
Time: O(log n)
Sourcepub fn update(&self, index: usize, value: A) -> Vector<A>
pub fn update(&self, index: usize, value: A) -> Vector<A>
Create a new vector with the value at index index updated.
Panics if the index is out of bounds.
Time: O(log n)
Examples
let mut vec = vector![1, 2, 3];
assert_eq!(vector![1, 5, 3], vec.update(1, 5));Sourcepub fn set(&mut self, index: usize, value: A) -> A
pub fn set(&mut self, index: usize, value: A) -> A
Update the value at index index in a vector.
Returns the previous value at the index.
Panics if the index is out of bounds.
Time: O(log n)
Sourcepub fn swap(&mut self, i: usize, j: usize)
pub fn swap(&mut self, i: usize, j: usize)
Swap the elements at indices i and j.
Time: O(log n)
Sourcepub fn push_front(&mut self, value: A)
pub fn push_front(&mut self, value: A)
Push a value to the front of a vector.
Time: O(1)*
Examples
let mut vec = vector![5, 6, 7];
vec.push_front(4);
assert_eq!(vector![4, 5, 6, 7], vec);Sourcepub fn push_back(&mut self, value: A)
pub fn push_back(&mut self, value: A)
Push a value to the back of a vector.
Time: O(1)*
Examples
let mut vec = vector![1, 2, 3];
vec.push_back(4);
assert_eq!(vector![1, 2, 3, 4], vec);Sourcepub fn pop_front(&mut self) -> Option<A>
pub fn pop_front(&mut self) -> Option<A>
Remove the first element from a vector and return it.
Time: O(1)*
Examples
let mut vec = vector![1, 2, 3];
assert_eq!(Some(1), vec.pop_front());
assert_eq!(vector![2, 3], vec);Sourcepub fn pop_back(&mut self) -> Option<A>
pub fn pop_back(&mut self) -> Option<A>
Remove the last element from a vector and return it.
Time: O(1)*
Examples
let mut vec = vector![1, 2, 3];
assert_eq!(Some(3), vec.pop_back());
assert_eq!(vector![1, 2], vec);Sourcepub fn append(&mut self, other: Vector<A>)
pub fn append(&mut self, other: Vector<A>)
Append the vector other to the end of the current vector.
Time: O(log n)
Examples
let mut vec = vector![1, 2, 3];
vec.append(vector![7, 8, 9]);
assert_eq!(vector![1, 2, 3, 7, 8, 9], vec);Sourcepub fn retain<F>(&mut self, f: F)
pub fn retain<F>(&mut self, f: F)
Retain only the elements specified by the predicate.
Remove all elements for which the provided function f
returns false from the vector.
Time: O(n)
Sourcepub fn split_off(&mut self, index: usize) -> Vector<A>
pub fn split_off(&mut self, index: usize) -> Vector<A>
Split a vector at a given index.
Split a vector at a given index, leaving the left hand side in the current vector and returning a new vector containing the right hand side.
Time: O(log n)
Examples
let mut left = vector![1, 2, 3, 7, 8, 9];
let right = left.split_off(3);
assert_eq!(vector![1, 2, 3], left);
assert_eq!(vector![7, 8, 9], right);Sourcepub fn skip(&self, count: usize) -> Vector<A>
pub fn skip(&self, count: usize) -> Vector<A>
Construct a vector with count elements removed from the
start of the current vector.
Time: O(log n)
Sourcepub fn take(&self, count: usize) -> Vector<A>
pub fn take(&self, count: usize) -> Vector<A>
Construct a vector of the first count elements from the
current vector.
Time: O(log n)
Sourcepub fn truncate(&mut self, len: usize)
pub fn truncate(&mut self, len: usize)
Truncate a vector to the given size.
Discards all elements in the vector beyond the given length.
Panics if the new length is greater than the current length.
Time: O(log n)
Sourcepub fn slice<R>(&mut self, range: R) -> Vector<A>where
R: RangeBounds<usize>,
pub fn slice<R>(&mut self, range: R) -> Vector<A>where
R: RangeBounds<usize>,
Extract a slice from a vector.
Remove the elements from start_index until end_index in
the current vector and return the removed slice as a new
vector.
Time: O(log n)
Sourcepub fn insert(&mut self, index: usize, value: A)
pub fn insert(&mut self, index: usize, value: A)
Insert an element into a vector.
Insert an element at position index, shifting all elements
after it to the right.
Performance Note
While push_front and push_back are heavily optimised
operations, insert in the middle of a vector requires a
split, a push, and an append. Thus, if you want to insert
many elements at the same location, instead of inserting
them one by one, you should rather create a new vector
containing the elements to insert, split the vector at the
insertion point, and append the left hand, the new vector and
the right hand in order.
Time: O(log n)
Sourcepub fn remove(&mut self, index: usize) -> A
pub fn remove(&mut self, index: usize) -> A
Remove an element from a vector.
Remove the element from position ‘index’, shifting all elements after it to the left, and return the removed element.
Performance Note
While pop_front and pop_back are heavily optimised
operations, remove in the middle of a vector requires a
split, a pop, and an append. Thus, if you want to remove many
elements from the same location, instead of removeing them
one by one, it is much better to use slice.
Time: O(log n)
Sourcepub fn insert_ord(&mut self, item: A)where
A: Ord,
pub fn insert_ord(&mut self, item: A)where
A: Ord,
Insert an element into a sorted vector.
Insert an element into a vector in sorted order, assuming the vector is already in sorted order.
Time: O(log n)
Examples
let mut vec = vector![1, 2, 3, 7, 8, 9];
vec.insert_ord(5);
assert_eq!(vector![1, 2, 3, 5, 7, 8, 9], vec);